<?php


namespace Service;

use Doctrine\ORM\EntityManager;
use Repository\AclLabels as AclLabelsRepository;
use Util\InstanceProvider;
use Zeedhi\Framework\DB\Mongo\Exception;
use Zeedhi\Framework\ORM\DateTime;

class AclLabels
{
    const COLLECTION_NAME = 'ACL_LABELS_LOG';
    /**
     * @var AclLabelsRepository
     */
    private $aclLabelsRepository;

    /**
     * @var EntityManager
     */
    private $entityManager;

    public function __construct(EntityManager $entityManager, AclLabelsRepository $aclLabelsRepository)
    {
        $this->aclLabelsRepository = $aclLabelsRepository;
        $this->entityManager = $entityManager;
    }

    /**
     * @param $projectId
     * @return array
     */
    public function findAclLabelsByProject($projectId): array
    {
        return $this->aclLabelsRepository->findAclLabelsByProject($projectId);
    }

    public function saveAclLabelsByProject($projectId, $labelsFile): array
    {
        $oldNode = '';
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Project $project */
        $project = InstanceProvider::getRepository()->getLocalProject()->find($projectId);

        /** @var \Zeedhi\DevOps\Model\Profile\Entities\AclLabels $aclLabels */
        $aclLabels = $this->aclLabelsRepository->findOneBy(array('project' => $project));

        if (empty($aclLabels)) {
            $aclLabels = InstanceProvider::getFactory()->getAclLabels()->createAclLabels($project, $labelsFile);
        } else {
            $oldNode = $aclLabels->getLabelsFile();
            $aclLabels->setLabelsFile($labelsFile);
        }

        $this->entityManager->persist($aclLabels);
        $this->entityManager->flush();

        if ($oldNode) {
            $this->saveLogMongo($oldNode, $aclLabels->getLabelsFile(), $aclLabels->getProject()->getId());
        }

        return [];
    }

    /**
     * @throws Exception
     */
    public function saveLogMongo($oldNode, $newNode, $projectId)
    {
        $datetimeNow = \DateTime::createFromFormat('Y-m-d H:i:s', (new \DateTime())->format('Y-m-d') . ' 00:00:00', new \DateTimeZone('UTC'));
        $logObject = array(
            'oldNode' => $oldNode,
            'newNode' => $newNode,
            'date' => InstanceProvider::getService()->getMongoManager()->convertDate($datetimeNow),
            'projectId' => intval($projectId),
            'cdoperador' => InstanceProvider::getLibEnvironment()->getCurrentUserId(),
            'nmoperador' => InstanceProvider::getLibEnvironment()->getCurrentUserName(),
        );
        InstanceProvider::getMongoDB()->insert(self::COLLECTION_NAME, $logObject);
    }

    /**
     * @param $date
     * @param $inFormat
     * @param $outFormat
     * @return string
     */
    private function getReversedDate($date, $inFormat = 'd/m/Y', $outFormat = 'Ymd'): string
    {
        $date = \DateTime::createFromFormat($inFormat, $date);
        return $date->format($outFormat);
    }

    /**
     * @param $projectId
     * @return array
     * @throws \Exception
     */
    public function findAclLabelsLogByProject($params): array
    {
        if (!empty($params['dynamic_filter'])) {
            $this->handleDynamicFilter($params);
        }
        $this->handleProjectIdFilter($params);
//        dd($params);
        $mongoManager = InstanceProvider::getService()->getMongoManager();
        return $mongoManager->findNext(self::COLLECTION_NAME, $params, true, false);
    }

    public function handleProjectIdFilter(&$params)
    {
        $params['dynamic_filter']['projectId'] = array(
            array(
                'operation' => 'EQUALS',
                'relation' => 'AND',
                'value' => intval($params['projectId'])
            )
        );
    }

    /**
     * @throws \Exception
     */
    private function handleDynamicFilter(&$params)
    {
        $params['dynamic_filter'] = json_decode(utf8_encode(base64_decode($params['dynamic_filter'])), true);
        if (!empty($params['dynamic_filter']['date'])) {
            foreach ($params['dynamic_filter']['date'] as &$i) {
                $i['value'][0] = InstanceProvider::getService()->getMongoManager()->convertDate(new \DateTime($i['value'][0]));
                if (empty($i['value'][1])) {
                    $i['value'][1] = $i['value'][0];
                } else {
                    $i['value'][1] = InstanceProvider::getService()->getMongoManager()->convertDate(new \DateTime($i['value'][1]));
                }
            }
        }
    }
}
