import { Metadata, Singleton } from '@zeedhi/core';
import { Modal, Select, Tree } from '@zeedhi/common';
import Store from '@/store';
import { PmLibDialogService, PmLibFormSave } from '@zeedhi/pm-lib';
import { ZdTextInput } from '@zeedhi/vuetify';
import { NodeType } from '@/services/AclLabels/NodeType';
import { AclLabelsService } from '@/services/AclLabels/AclLabelsService';
import Messages from '@/services/Messages';

@Singleton
export class AclLabelsAddLabelModalController {
	get addLabelModal(): Modal {
		return Store.getters['AclLabels/getAddLabelModal'];
	}

	get aclLabelsTreeComponent(): Tree {
		return Store.getters['AclLabels/getAclLabelsTreeComponent'];
	}

	get nodeTypeValue() {
		return Store.getters['AclLabels/getNodeTypeValue'];
	}

	get hasChanges(): boolean {
		return Store.getters['AclLabels/getHasChanges'];
	}

	set hasChanges(value: boolean) {
		Store.commit('AclLabels/setHasChanges', value);
	}

	get version(): number {
		return Store.getters['AclLabels/getVersion'];
	}

	set version(value: number) {
		Store.commit('AclLabels/setVersion', value);
	}

	public onMountedAddLabelModalForm() {
		const addLabelForm = Metadata.getInstance<PmLibFormSave>('addLabelModalForm');
		const typeField = addLabelForm.getChildInstance<Select>('type');
		const labelField = addLabelForm.getChildInstance<ZdTextInput>('label');
		labelField.readonly = this.nodeTypeValue === NodeType.grouped;
		typeField.setValue(this.nodeTypeValue);
	}

	public closeAddLabelModalForm() {
		this.addLabelModal?.hide();
	}

	public saveAddLabelModalForm() {
		const addLabelForm = Metadata.getInstance<PmLibFormSave>('addLabelModalForm');
		if (addLabelForm.validate() && this.validateDuplicateLabels(addLabelForm)) {
			const newItem = this.getNewItem(addLabelForm);
			this.aclLabelsTreeComponent.insertNode(newItem);
			this.hasChanges = true;
			this.addLabelModal?.hide();
		}
	}

	private validateDuplicateLabels(addLabelForm: PmLibFormSave): boolean {
		if (addLabelForm.value.type === NodeType.rule) {
			const { label } = addLabelForm.value;
			const allLabels = AclLabelsService.getAllLabelsInNodes(this.aclLabelsTreeComponent.nodes);
			if (allLabels.includes(label)) {
				PmLibDialogService.showError(Messages.MSG_DUPLICATE_LABEL_SAVE);
			}
			return !allLabels.includes(label);
		}

		return true;
	}

	public getNewItem(form: PmLibFormSave): any {
		const newItem: any = {
			title: form.value.name,
			data: {
				version: this.version + 1,
				type: form.value.type,
			},
		};

		if (form.value.label && form.value.type !== NodeType.grouped) {
			newItem.data.label = form.value.label;
		}

		return newItem;
	}
}
