import { Loader, Metadata, Singleton } from '@zeedhi/core';
import { IGridEvent } from '@zeedhi/common';
import Store from '@/store';
import { TekGrid, TekRestDatasource } from '@zeedhi/teknisa-components-common';
import { PmLibUtil } from '@zeedhi/pm-lib';
import moment from 'moment';
import { AclLabelsLogDetailController } from '@/controllers/AclLabels/AclLabelsLogDetailController';

@Singleton
export class AclLabelsLogGridController {
	public currentProject: any = {};

	public projectName: string = '';

	get gridIsVisible() {
		return Store.getters['AclLabels/getLogGridIsVisible'];
	}

	get currentProjectId() {
		return Store.getters['AclLabels/getCurrentProject'].PROJECT_ID;
	}

	public async onClickTab() {
		await PmLibUtil.timeout(500);
		const grid = Metadata.getInstance<TekGrid>('aclLabelsLogGrid');
		grid.datasource.filter = { projectId: this.currentProjectId };
		const initDate = moment().subtract(30, 'd').format('YYYY-MM-DD');
		const endDate = moment().format('YYYY-MM-DD');
		if (grid.datasource instanceof TekRestDatasource) {
			grid.datasource.dynamicFilter = {
				date: [
					{
						operation: 'BETWEEN',
						relation: 'AND',
						value: [initDate, endDate],
					},
				],
			};
		}
		await grid.reload();
	}

	public async rowClickAclLabelsLogGrid({ row }: IGridEvent) {
		Store.commit('AclLabels/setAclLabelsLogRow', row);
		this.onLoadNodeInDetail();
		Store.commit('AclLabels/setLogGridIsVisible', false);
	}

	private onLoadNodeInDetail() {
		const aclLabelsLogDetailController: AclLabelsLogDetailController = Loader.getInstance('AclLabelsLogDetailController');
		aclLabelsLogDetailController.onLoadNodes();
	}
}
