import { PmLibHttp, PmLibUtil } from '@zeedhi/pm-lib';
import { Metadata } from '@zeedhi/core';
import { Modal, ModalService, Tree } from '@zeedhi/common';

interface IState {
	aclLabelsTreeComponent: Tree | null,
	addLabelModal: Modal | null,
	importMenuModal: Modal | null,
	duplicateLabelsModal: Modal | null,
	[key: string]: any;
}

const state: IState = {
	gridIsVisible: true,
	currentProject: {},
	version: 0,
	originalNodes: [],
	aclLabelsTreeComponent: null,
	hasChanges: false,
	nodeTypeValue: 'GROUPED',
	addLabelModal: null,
	importMenuModal: null,
	duplicateLabelsModal: null,
	logGridIsVisible: true,
	aclLabelsLogRow: {},
};

const mutations = {
	setCurrentProject: (_state: any, payload: any) => {
		_state.currentProject = payload;
	},
	setGridIsVisible: (_state: any, payload: boolean) => {
		_state.gridIsVisible = payload;
	},
	setOriginalNodes: (_state: any, payload: Array<any>) => {
		_state.originalNodes = payload;
	},
	setAclLabelsTreeComponent: (_state: any, payload: Tree) => {
		_state.aclLabelsTreeComponent = payload;
	},
	setNodeTypeValue: (_state: any, payload: string) => {
		_state.nodeTypeValue = payload;
	},
	setAddLabelModal: (_state: any, payload: Modal) => {
		_state.addLabelModal = payload;
	},
	setImportMenuModal: (_state: any, payload: Modal) => {
		_state.importMenuModal = payload;
	},
	setDuplicateLabelsModal: (_state: any, payload: Modal) => {
		_state.duplicateLabelsModal = payload;
	},
	setHasChanges: (_state: any, payload: boolean) => {
		_state.hasChanges = payload;
	},
	setVersion: (_state: any, payload: number) => {
		_state.version = payload;
	},
	setLogGridIsVisible: (_state: any, payload: boolean) => {
		_state.logGridIsVisible = payload;
	},
	setAclLabelsLogRow: (_state: any, payload: any) => {
		_state.aclLabelsLogRow = payload;
	},
};

const actions = {
	loadNodes: async (context: any, projectId: string | number) => {
		const { data: { data } } = await PmLibHttp.$get('/projects/aclLabels', { params: { projectId } });
		const originalNodes = data.length ? JSON.parse(data[0].LABELS_FILE) : [];
		context.commit('setOriginalNodes', originalNodes);
		context.state.aclLabelsTreeComponent.nodes = PmLibUtil.clone(originalNodes);
	},
	loadAclLabelsTreeComponent: async (context: any) => {
		const aclLabelsTreeComponent = Metadata.getInstance<Tree>('aclLabelsTree');
		context.commit('setAclLabelsTreeComponent', aclLabelsTreeComponent);
	},
	loadAddLabelModal: async (context: any) => {
		const addLabelModalPath = 'aclLabels/components/detail/tabs/labels/modais/addLabel';
		const addLabelModal = await ModalService.createFromJson(addLabelModalPath, true);
		context.commit('setAddLabelModal', addLabelModal);
	},
	loadImportMenuModal: async (context: any) => {
		const importMenuModalPath = 'aclLabels/components/detail/tabs/labels/modais/importMenu';
		const importMenuModal = await ModalService.createFromJson(importMenuModalPath, true);
		context.commit('setImportMenuModal', importMenuModal);
	},
	loadDuplicateLabelsModal: async (context: any) => {
		const duplicateLabelsModalPath = 'aclLabels/components/detail/tabs/labels/modais/duplicateLabels';
		const duplicateLabelsModal = await ModalService.createFromJson(duplicateLabelsModalPath, true);
		context.commit('setDuplicateLabelsModal', duplicateLabelsModal);
	},
};

const getters = {
	getCurrentProject: (_state: any) => _state.currentProject,
	getCurrentProjectName: (_state: any) => _state.currentProject.PROJECT_NAME ?? '',
	getGridIsVisible: (_state: any) => _state.gridIsVisible,
	getOriginalNodes: (_state: any) => _state.originalNodes,
	getAclLabelsTreeComponent: (_state: any) => _state.aclLabelsTreeComponent,
	getNodeTypeValue: (_state: any) => _state.nodeTypeValue,
	getAddLabelModal: (_state: any) => _state.addLabelModal,
	getImportMenuModal: (_state: any) => _state.importMenuModal,
	getDuplicateLabelsModal: (_state: any) => _state.duplicateLabelsModal,
	getHasChanges: (_state: any) => _state.hasChanges,
	getVersion: (_state: any) => _state.version,
	getLogGridIsVisible: (_state: any) => _state.logGridIsVisible,
	getAclLabelsLogRow: (_state: any) => _state.aclLabelsLogRow,
};

export default {
	namespaced: true,
	state,
	mutations,
	actions,
	getters,
};
